﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'


Public Class TypeComparer
	Implements IComparer

	Public Sub New()
		_comparer = New Comparer(System.Globalization.CultureInfo.InvariantCulture)
	End Sub

	Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements IComparer.Compare
		Dim xx As Type = TryCast(x, Type)
		Dim yy As Type = TryCast(y, Type)

		Dim xPriority As Integer = GetPriority(xx)
		Dim yPriority As Integer = GetPriority(yy)

		If xPriority < yPriority Then
			Return -1
		End If

		If xPriority > yPriority Then
			Return 1
		End If

		Return _comparer.Compare(xx.Name, yy.Name)
	End Function

	Private Function GetPriority(ByVal type As Type) As Integer
		' Classes first
		If type.IsClass Then
			' Delegates last
			If type.IsSubclassOf(GetType([Delegate])) Then
				Return 3
			End If

			Return 0
		End If

		' Interfaces next
		If type.IsInterface Then
			Return 1
		End If

		' Structs next
		If type.IsValueType Then
			Return 2
		End If

		Return 4
	End Function


	Private _comparer As Comparer

End Class